package com.supervision.knowsub.controller.knowledge;

import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.knowsub.entity.vo.knowledge.KnowledgeDetailResVO;
import com.supervision.knowsub.entity.vo.knowledge.ModifyKnowledgeReqVO;
import com.supervision.knowsub.entity.vo.knowledge.SaveKnowledgeReqVO;
import com.supervision.knowsub.service.KnowledgeManageService;
import com.supervision.knowsub.model.Knowledge;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;

@Slf4j
@RestController
@Tag(name = "知识库查询")
@RequiredArgsConstructor
@RequestMapping("knowledge")
public class KnowledgeManageController {

    private final KnowledgeManageService knowledgeManageService;

    @Operation(summary = "查询知识库分页")
    @Parameters({
            @Parameter(name = "status", description = "流转状态 为空查全部,1草稿 2通过 3驳回 4已撤回 5已失效 6已删除 10待审批(新增) 11待审批(删除) 12待审批(撤回)", in = ParameterIn.QUERY),
            @Parameter(name = "title", description = "标题,支持模糊", in = ParameterIn.QUERY),
            @Parameter(name = "publishDeptId", description = "发文部门id", in = ParameterIn.QUERY),
            @Parameter(name = "baseId", description = "应用字库ID", in = ParameterIn.QUERY),
            @Parameter(name = "submittedDeptId", description = "报送部门ID", in = ParameterIn.QUERY),
            @Parameter(name = "publishDateBegin", description = "发文时间开始", in = ParameterIn.QUERY),
            @Parameter(name = "publishDateEnd", description = "发文时间结束", in = ParameterIn.QUERY),
            @Parameter(name = "pageNum", description = "分页", in = ParameterIn.QUERY),
            @Parameter(name = "pageSize", description = "分页", in = ParameterIn.QUERY)
    })
    @GetMapping("queryKnowledgePage")
    public IPage<Knowledge> queryKnowledgePage(@RequestParam(required = false) Integer status,
                                               @RequestParam(required = false) String title,
                                               @RequestParam(required = false) String publishDeptId,
                                               @RequestParam(required = false) String baseId,
                                               @RequestParam(required = false) String submittedDeptId,
                                               @RequestParam(required = false)LocalDateTime publishDateBegin,
                                               @RequestParam(required = false)LocalDateTime publishDateEnd,
                                               Integer pageNum, Integer pageSize) {
        return knowledgeManageService.queryKnowledgePage(status, title, publishDeptId, baseId, submittedDeptId, publishDateBegin, publishDateEnd, pageNum, pageSize);
    }

    @Operation(summary = "新增知识库")
    @PostMapping("saveKnowledge")
    public void saveKnowledge(@RequestBody SaveKnowledgeReqVO reqVO) {
        knowledgeManageService.saveKnowledge(reqVO);
    }

    @Operation(summary = "修改知识库(已撤回才能编辑)")
    @PostMapping("updateKnowledge")
    public void updateKnowledge(@RequestBody ModifyKnowledgeReqVO reqVO) {
        Assert.notBlank(reqVO.getKnowledgeId(), "知识ID不能为空");
        knowledgeManageService.updateKnowledge(reqVO);
    }

    @Operation(summary = "撤回知识库")
    @GetMapping("recallKnowledge")
    public void recallKnowledge(String knowledgeId) {
        knowledgeManageService.recallKnowledge(knowledgeId);
    }

    @Operation(summary = "查看知识的详细信息")
    @GetMapping("queryKnowledgeDetail")
    public KnowledgeDetailResVO queryKnowledgeDetail(String knowledgeId) {
        return knowledgeManageService.queryKnowledgeDetail(knowledgeId);
    }

    @Operation(summary = "删除知识库-逻辑删除")
    @GetMapping("deleteKnowledge")
    public void deleteKnowledge(String knowledgeId) {

    }


}
