package com.supervision.knowsub.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.knowsub.dto.flow.FlowBaseRelationTypeDTO;
import com.supervision.knowsub.exception.BusinessException;
import com.supervision.knowsub.model.SystemBase;
import com.supervision.knowsub.model.SystemFlow;
import com.supervision.knowsub.model.SystemFlowBaseRelation;
import com.supervision.knowsub.model.SystemFlowTypeRelation;
import com.supervision.knowsub.service.*;
import com.supervision.knowsub.vo.flow.BaseResVo;
import com.supervision.knowsub.vo.flow.FlowInfoReqVo;
import com.supervision.knowsub.vo.flow.FlowInfoResVo;
import com.supervision.knowsub.vo.sublibrary.SubLibraryResVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

@Service
@Slf4j
@RequiredArgsConstructor
public class FlowManageServiceImpl implements FlowManageService {

    private final SystemFlowService systemFlowService;

    private final SystemFlowBaseRelationService systemFlowBaseRelationService;

    private final SystemFlowTypeRelationService systemFlowTypeRelationService;

    private final SystemFlowRuleService systemFlowRuleService;


    private final ApplicationSubLibraryService applicationSubLibraryService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String saveFlow(FlowInfoReqVo flowInfoReqVo) {

        assertBase(flowInfoReqVo);

        //  一个子库最多对应一种流程类型
        List<String> baseIdList = flowInfoReqVo.getBaseIdList();
        if (CollUtil.isNotEmpty(baseIdList)){
            List<SystemFlowTypeRelation> flowTypeRelationList = systemFlowTypeRelationService.lambdaQuery().in(SystemFlowTypeRelation::getBaseId, baseIdList).list();
            List<String> usedBaseIds = listUsedBaseIds(flowInfoReqVo.getFlowTypeList());
            flowTypeRelationList.forEach(flowTypeRelation ->
                    Assert.isTrue(!usedBaseIds.contains(flowTypeRelation.getBaseId()), "该子库已存在流程类型"));
        }

        // 保存数据
        return doSaveFlow(flowInfoReqVo);
    }

    private String doSaveFlow(FlowInfoReqVo flowInfoReqVo) {
        // 保存主表数据
        SystemFlow systemFlow = new SystemFlow();
        systemFlow.setFlowName(flowInfoReqVo.getFlowName());
        systemFlow.setRemark(flowInfoReqVo.getRemark());
        systemFlowService.save(systemFlow);

        // 保存关联表数据
        List<String> baseIdList = flowInfoReqVo.getBaseIdList();
        List<SystemFlowBaseRelation> flowBaseRelationList = flowInfoReqVo.getBaseIdList().stream().map(baseId -> {
            SystemFlowBaseRelation flowBaseRelation = new SystemFlowBaseRelation();
            flowBaseRelation.setFlowId(systemFlow.getId());
            flowBaseRelation.setBaseId(baseId);
            return flowBaseRelation;
        }).toList();
        systemFlowBaseRelationService.saveBatch(flowBaseRelationList);

        List<SystemFlowTypeRelation> flowTypeRelationList = flowInfoReqVo.getFlowTypeList().stream().map(flowType -> baseIdList.stream().map(baseId -> {
            SystemFlowTypeRelation flowTypeRelation = new SystemFlowTypeRelation();
            flowTypeRelation.setFlowType(flowType);
            flowTypeRelation.setBaseId(baseId);
            flowTypeRelation.setFlowId(systemFlow.getId());
            return flowTypeRelation;
        }).toList()).flatMap(Collection::stream).toList();
        systemFlowTypeRelationService.saveBatch(flowTypeRelationList);

        return systemFlow.getId();
    }

    @Override
    public List<String> listUsedBaseIds(List<Integer> typeIds){
        if (CollUtil.isEmpty(typeIds)){
            return CollUtil.newArrayList();
        }

        return systemFlowTypeRelationService.lambdaQuery().in(SystemFlowTypeRelation::getFlowId, typeIds).list()
                .stream().map(SystemFlowTypeRelation::getBaseId).distinct().toList();
    }

    private void assertBase(FlowInfoReqVo flowInfoReqVo){
        Assert.notEmpty(flowInfoReqVo.getFlowName());
        Assert.notEmpty(flowInfoReqVo.getFlowTypeList());
        Assert.notEmpty(flowInfoReqVo.getBaseIdList());
        Assert.notEmpty(flowInfoReqVo.getNodeInfoList());
    }
    @Override
    public void updateFlow(FlowInfoReqVo flowInfoReqVo) {

    }

    @Override
    public Boolean deleteFlow(String id) {
        throw new BusinessException("暂不支持删除流程");
    }

    @Override
    public IPage<FlowInfoResVo> pageListFlow(String flowName, Integer pageNum, Integer pageSize) {

        return systemFlowService.pageListFlow(flowName, pageNum, pageSize);
    }

    @Override
    public List<BaseResVo> optionalBaseList(List<Integer> flowType) {
        List<SystemBase> allSubLibrary = applicationSubLibraryService.listAllSubLibrary();

        final List<SystemFlowTypeRelation> flowTypeRelationList = new ArrayList<>();
        if (CollUtil.isNotEmpty(flowType)){
            flowTypeRelationList.addAll(
                    systemFlowTypeRelationService.lambdaQuery()
                            .in(SystemFlowTypeRelation::getFlowType, flowType).list()) ;
        }

        return allSubLibrary.stream().filter(systemBase ->
                flowTypeRelationList.stream().noneMatch(flowTypeRelation ->
                        flowTypeRelation.getBaseId().equals(systemBase.getId()))).map(systemBase -> {
                            BaseResVo baseResVo = new BaseResVo();
                            baseResVo.setBaseId(systemBase.getId());
                            baseResVo.setBaseName(systemBase.getBaseName());
                            return baseResVo;
        }).toList();
    }
}
